# -*- coding: utf-8 -*-

"""
DSuper8 project based on Joe Herman's rpi-film-capture.

Software modified by Manuel Ángel.

User interface redesigned by Manuel Ángel.

DS8Config.py: Module to save the configuration and restore it from a file.

Latest version: July 2021.
"""

from configparser import ConfigParser

from pathlib import Path

from logging import info


class DS8ConfigParser(ConfigParser):

    def updateConfigFromUI(self, ui):
        if not self.has_section("Start"):
            self.add_section("Start")
        if not self.has_section("Camera"):
            self.add_section("Camera")
        if not self.has_section("Capture"):
            self.add_section("Capture")
        if not self.has_section("Advanced"):
            self.add_section("Advanced")
        if not self.has_section("Post-capture"):
            self.add_section("Post-capture")
        if not self.has_section("HDR"):
            self.add_section("HDR")

        sec = "Start"
        self.set(sec, "zoomdial", str(ui.zoomDial.value()))
        self.set(sec, "roix", str(ui.roiX))
        self.set(sec, "roiy", str(ui.roiY))
        self.set(sec, "lightcheckbox", str(ui.lightCheckbox.isChecked()))
        self.set(sec, "startposcheckbox", str(ui.startPosCheckBox.isChecked()))

        sec = "Camera"
        self.set(sec, "iso", str(ui.ISOcomboBox.currentIndex()))
        self.set(sec, "ev", str(ui.EV.value()))
        self.set(sec, "awbbox", str(ui.awbBox.currentIndex()))
        self.set(sec, "bluegain", str(ui.blueGainBox.value()))
        self.set(sec, "redgain", str(ui.redGainBox.value()))
        self.set(sec, "manualbluegain", str(ui.manualBlueGain))
        self.set(sec, "manualredgain", str(ui.manualRedGain))
        self.set(sec, "brightness", str(ui.brightnessBox.value()))
        self.set(sec, "contrast", str(ui.contrastBox.value()))
        self.set(sec, "saturation", str(ui.saturationBox.value()))

        sec = "Capture"
        self.set(sec, "endframebox", str(ui.endFrameBox.value()))
        self.set(sec, "bracketingbox", str(ui.bracketingBox.value()))
        self.set(sec, "stopsbox", str(ui.stopsBox.value()))
        self.set(sec, "autoexpcheckbox", str(ui.autoExpCheckBox.isChecked()))
        self.set(sec, "timeexpbox", str(ui.timeExpBox.value()))
        self.set(sec, "capfolderbox", ui.capFolderBox.text())
        self.set(sec, "framelcd", str(int(ui.frameLcd.value())))
        self.set(sec, "nextframebox", str(ui.nextFrameBox.value()))

        sec = "Advanced"
        self.set(sec, "vflip", str(ui.vflip.isChecked()))
        self.set(sec, "hflip", str(ui.hflip.isChecked()))
        self.set(sec, "bw", str(ui.bw.isChecked()))
        self.set(sec, "drc", str(ui.drcBox.currentIndex()))
        self.set(sec, "sharpnessbox", str(ui.sharpnessBox.value()))
        self.set(sec, "resolutionbox", str(ui.resolutionBox.currentIndex()))
        self.set(sec, "stillcap", str(ui.stillCap.isChecked()))

        sec = "Post-capture"
        self.set(sec, "showhist", str(ui.showHist.isChecked()))
        self.set(sec, "logarithmhist", str(ui.logarithmHist.isChecked()))
        self.set(sec, "roundcorns", str(ui.roundCorns.isChecked()))
        self.set(sec, "rotation", str(ui.rotation.isChecked()))
        self.set(sec, "rotationbox", str(ui.rotationBox.value()))
        self.set(sec, "cropping", str(ui.cropping.isChecked()))
        self.set(sec, "croptopbox", str(ui.cropTopBox.value()))
        self.set(sec, "cropleftbox", str(ui.cropLeftBox.value()))
        self.set(sec, "croprightbox", str(ui.cropRightBox.value()))
        self.set(sec, "cropbottombox", str(ui.cropBottomBox.value()))

        sec = "HDR"
        self.set(sec, "hdrmertens", str(ui.HDRMertensRadioButton.isChecked()))
        self.set(sec, "hdrdebevec", str(ui.HDRDebevecRadioButton.isChecked()))
        self.set(sec, "simple", str(ui.SimpleRadioButton.isChecked()))
        self.set(sec, "simplegammaspinbox", str(ui.SimpleGammaSpinBox.value()))
        self.set(sec, "reinhard", str(ui.ReinhardRadioButton.isChecked()))
        self.set(sec, "reinhardgammaspinbox", str(ui.ReinhardGammaSpinBox.
                                            value()))
        self.set(sec, "reinhardintensityspinbox", str(ui.
                                                ReinhardIntensitySpinBox.
                                                value()))
        self.set(sec, "reinhardlightspinbox", str(ui.ReinhardLightSpinBox.
                                            value()))
        self.set(sec, "reinhardcolorspinbox", str(ui.ReinhardColorSpinBox.
                                            value()))
        self.set(sec, "drago", str(ui.DragoRadioButton.isChecked()))
        self.set(sec, "dragogammaspinbox", str(ui.DragoGammaSpinBox.value()))
        self.set(sec, "dragosaturationspinbox", str(ui.DragoSaturationSpinBox.
                                              value()))
        self.set(sec, "dragobiasspinbox", str(ui.DragoBiasSpinBox.value()))
        self.set(sec, "mantiuk", str(ui.MantiukRadioButton.isChecked()))
        self.set(sec, "mantiukgammaspinbox", str(ui.MantiukGammaSpinBox.
                                           value()))
        self.set(sec, "mantiuksaturationspinbox", str(ui.
                                                MantiukSaturationSpinBox.
                                                value()))
        self.set(sec, "mantiukscalespinbox", str(ui.MantiukScaleSpinBox.
                                           value()))

    def updateUIfromConfig(self, ui):
        sec = "Start"
        ui.zoomDial.setValue(self.getint(sec, "zoomdial"))
        ui.roiX = (self.getint(sec, "roix"))
        ui.roiY = (self.getint(sec, "roiy"))
        ui.lightCheckbox.setChecked(self.getboolean(sec, "lightcheckbox"))

        sec = "Camera"
        ui.ISOcomboBox.setCurrentIndex(self.getint(sec, "iso"))
        ui.EV.setValue(self.getint(sec, "ev"))
        ui.awbBox.setCurrentIndex(self.getint(sec, "awbbox"))
        ui.blueGainBox.setValue(self.getint(sec, "bluegain"))
        ui.redGainBox.setValue(self.getint(sec, "redgain"))
        ui.manualBlueGain = (self.getint(sec, "manualbluegain"))
        ui.manualRedGain = (self.getint(sec, "manualredgain"))
        ui.brightnessBox.setValue(self.getint(sec, "brightness"))
        ui.contrastBox.setValue(self.getint(sec, "contrast"))
        ui.saturationBox.setValue(self.getint(sec, "saturation"))

        sec = "Capture"
        ui.endFrameBox.setValue(self.getint(sec, "endframebox"))
        ui.bracketingBox.setValue(self.getint(sec, "bracketingbox"))
        ui.stopsBox.setValue(self.getfloat(sec, "stopsbox"))
        ui.autoExpCheckBox.setChecked(self.getboolean(sec, "autoexpcheckbox"))
        ui.timeExpBox.setValue(self.getfloat(sec, "timeexpbox"))
        capFolder = self.get(sec, "capfolderbox").strip()
        if capFolder and Path(capFolder).is_dir():
            ui.capFolderBox.setText(capFolder)
        else:
            ui.capFolderBox.setText("")
            info("The folder " + capFolder + " does not exist")
        ui.frameLcd.display(self.getint(sec, "framelcd"))
        ui.nextFrameBox.setValue(self.getint(sec, "nextframebox"))

        sec = "Advanced"
        ui.vflip.setChecked(self.getboolean(sec, "vflip"))
        ui.hflip.setChecked(self.getboolean(sec, "hflip"))
        ui.bw.setChecked(self.getboolean(sec, "bw"))
        ui.drcBox.setCurrentIndex(self.getint(sec, "drc"))
        ui.sharpnessBox.setValue(self.getint(sec, "sharpnessbox"))
        ui.resolutionBox.setCurrentIndex(self.getint(sec, "resolutionbox"))
        ui.stillCap.setChecked(self.getboolean(sec, "stillcap"))

        sec = "Post-capture"
        ui.showHist.setChecked(self.getboolean(sec, "showhist"))
        ui.logarithmHist.setChecked(self.getboolean(sec, "logarithmhist"))
        ui.roundCorns.setChecked(self.getboolean(sec, "roundcorns"))
        ui.rotation.setChecked(self.getboolean(sec, "rotation"))
        ui.rotationBox.setValue(self.getfloat(sec, "rotationbox"))
        ui.cropping.setChecked(self.getboolean(sec, "cropping"))
        ui.cropTopBox.setValue(self.getint(sec, "croptopbox"))
        ui.cropLeftBox.setValue(self.getint(sec, "cropleftbox"))
        ui.cropRightBox.setValue(self.getint(sec, "croprightbox"))
        ui.cropBottomBox.setValue(self.getint(sec, "cropbottombox"))

        sec = "HDR"
        ui.HDRMertensRadioButton.setChecked(self.getboolean(sec, "hdrmertens"))
        ui.HDRDebevecRadioButton.setChecked(self.getboolean(sec, "hdrdebevec"))
        ui.SimpleRadioButton.setChecked(self.getboolean(sec, "simple"))
        ui.SimpleGammaSpinBox.setValue(self.getfloat(sec, "simplegammaspinbox"))
        ui.ReinhardRadioButton.setChecked(self.getboolean(sec, "reinhard"))
        ui.ReinhardGammaSpinBox.setValue(self.getfloat(sec,
                                                 "reinhardgammaspinbox"))
        ui.ReinhardIntensitySpinBox.setValue(self.getfloat(sec,
                                             "reinhardintensityspinbox"))
        ui.ReinhardLightSpinBox.setValue(self.getfloat(sec,
                                                 "reinhardlightspinbox"))
        ui.ReinhardColorSpinBox.setValue(self.getfloat(sec,
                                                 "reinhardcolorspinbox"))
        ui.DragoRadioButton.setChecked(self.getboolean(sec, "drago"))
        ui.DragoGammaSpinBox.setValue(self.getfloat(sec, "dragogammaspinbox"))
        ui.DragoSaturationSpinBox.setValue(self.getfloat(sec,
                                                   "dragosaturationspinbox"))
        ui.DragoBiasSpinBox.setValue(self.getfloat(sec, "dragobiasspinbox"))
        ui.MantiukRadioButton.setChecked(self.getboolean(sec, "mantiuk"))
        ui.MantiukGammaSpinBox.setValue(self.getfloat(sec,
                                                "mantiukgammaspinbox"))
        ui.MantiukSaturationSpinBox.setValue(self.getfloat(sec,
                                             "mantiuksaturationspinbox"))
        ui.MantiukScaleSpinBox.setValue(self.getfloat(sec,
                                                "mantiukscalespinbox"))
                
                # It is configured last so that the rest of the settings are
                # made.
        sec = "Start"
        ui.startPosCheckBox.setChecked(self.getboolean(sec, "startposcheckbox"))

    # This function is to check the validity of the configuration file.
    # The existence and not duplication of all the sections and options of
    # the file is checked.
    # It is also checked that there are no options without associated value.
    # The function returns False if it encounters any errors.

    def validateConfigFile(self):

        ok = True

        # There must be 6 sections.
        if len(self.sections()) != 6:
            ok = False

        # We check the existence of all sections.
        if ok:
            for section in ["Start", "Camera", "Capture", "Advanced",
                            "Post-capture", "HDR"]:
                if section not in self.sections():
                    ok = False
                    break

        # We check the existence of all options.
        if ok and len(self.options("Start")) != 5:
            ok = False

        if ok:
            for option in ["zoomdial", "roix", "roiy", "lightcheckbox",
                           "startposcheckbox"]:
                if (option not in self.options("Start")
                    or self.get("Start", option).strip() == ""):
                    ok = False
                    break

        if ok and len(self.options("Camera")) != 10:
            ok = False

        if ok:
            for option in ["iso", "ev", "awbbox", "bluegain", "redgain",
                           "manualbluegain", "manualredgain", "brightness",
                           "contrast", "saturation"]:
                if (option not in self.options("Camera") or
                    self.get("Camera", option).strip() == ""):
                    ok = False
                    break

        if ok and len(self.options("Capture")) != 8:
            ok = False

        if ok:
            for option in ["endframebox", "bracketingbox",
                           "stopsbox", "autoexpcheckbox", "timeexpbox",
                           "capfolderbox", "framelcd", "nextframebox"]:
                if option not in self.options("Capture"):
                    ok = False
                    break
        # The "capfolderbox" option is allowed to be blank.
        if ok:
            for option in ["endframebox", "bracketingbox",
                           "stopsbox", "autoexpcheckbox", "timeexpbox",
                           "framelcd", "nextframebox"]:            
                if self.get("Capture", option).strip() == "":                
                    ok = False
                    break

        if ok and len(self.options("Advanced")) != 7:
            ok = False
                        
        if ok:
            for option in ["vflip", "hflip", "bw", "drc", "sharpnessbox",
                           "resolutionbox", "stillcap"]:
                if (option not in self.options("Advanced")
                    or self.get("Advanced", option).strip() == ""):
                    ok = False                    
                    break
                
        if ok and len(self.options("Post-capture")) != 10:
            ok = False            
        
        if ok:    
            for option in ["showhist", "logarithmhist", "roundcorns",
                           "rotation", "rotationbox", "cropping", "croptopbox",
                           "cropleftbox", "croprightbox", "cropbottombox"]:
                if (option not in self.options("Post-capture")
                    or self.get("Post-capture", option).strip() == ""):
                    ok = False                    
                    break
               
        if ok and len(self.options("HDR")) != 17:
            ok = False            
           
        if ok:
            for option in ["hdrmertens", "hdrdebevec", "simple",
                           "simplegammaspinbox", "reinhard",
                           "reinhardgammaspinbox", "reinhardintensityspinbox",
                           "reinhardlightspinbox", "reinhardcolorspinbox",
                           "drago", "dragogammaspinbox",
                           "dragosaturationspinbox", "dragobiasspinbox",
                           "mantiuk", "mantiukgammaspinbox",
                           "mantiuksaturationspinbox", "mantiukscalespinbox"]:
                if (option not in self.options("HDR")
                    or self.get("HDR", option).strip == ""):
                    ok = False                    
                    break
               
        return ok
