# -*- coding: utf-8 -*-

"""
DSuper8 project based on Joe Herman's rpi-film-capture.

Software modified by Manuel Ángel.

User interface redesigned by Manuel Ángel.

config.py: Configuration module and global variables of the client software.

Latest version: July 2021.
"""

from PyQt5.QtCore import QFile, QIODevice

from numpy import float32, fromstring, uint8, ndarray

from cv2 import imdecode, IMREAD_COLOR

from logging import info

# Resource file: default configuration file, splash and rounding masks for
# image angles.
import qrc_DSuper8

# Type of camera used.
# Comment on the one that does not apply.
# cameraType = "V1"
cameraType = "HQ"

# IP address of the server.
server_ip = "HAL9003.maodog.es"
# Monitor H resolution. Used to position the UI.
monitor_res = 1920
# Program output signal.
exitFlag = False
# Preview images indicator.
prevOn = False
# Test Image Indicator.
testImg = False
# Capture indicator.
captureOn = False
# Latest system state.
# It is used in the image window naming.
# P -> Preview
# T -> Test
# C -> Capture
lastMode = ""
# Socket for sending commands to the server.
ctrl_conn = False
# Film position. Frame number from the first.
frame_number = 1
# Limit number of frames to digitize.
frame_limit = 3600
# Image file naming index.
file_number = 1
# Image number received from server.
numImgRec = 1

# Base directory for configuration files and capture folder.
default_folder = "/home/mao/Super8/"

# Directory where the generated image files will be saved.
capFolder = "/home/mao/Super8/"

# File to save settings.
configfile = "/home/mao/Super8/DSuper8.conf"

# The title of the image window can be anything.
imgWinTitle = "DSuper8"

# Image Window Geometry.
# The size of the window does not affect the size of the capture images.
imgWinWidth = 864
imgWinX = 20
imgWinHeight = 648
imgWinY = 10

# Maximum dimensions of the final image. It is applied in the scaling of the captured
# image.
imgCapFinalW = 1920
imgCapFinalH = 1080

# Dimensions of the image initially captured by the camera, once the cutouts have
# been applied. Obtained in the imageResize function of DS8ImgThread.
imgCapIniW = 2028
imgCapIniH = 1520

# Dimensions of the resulting image once scaled. Obtained in the displayImg
# function of DS8ImgDialog.
imgCapResW = 1920
imgCapResH = 1080

# Exposure time matrix used by the HDR Debevec algorithm.
exposureTimes = ndarray([], dtype=float32)

# HDR image creation algorithm.
# It can be Mertens or Debevec.
blender = "Mertens"

# Tone mapping algorithm.
# It can be Simple, Reinhard, Drago or Mantiuk.
toneMap = "Simple"

# Parameters apply in the Simple tone mapping algorithm.
SimpleGamma = 1.0

# Parameters apply in Reinhard's tone mapping algorithm.
ReinhardGamma = 1.0
ReinhardIntensity = 0.0
ReinhardLight = 0.0
ReinhardColor = 1.0

# Parameters apply in the Drago tone mapping algorithm.
DragoGamma = 1.0
DragoSaturation = 0.0
DragoBias = 0.85

# Parameters apply in the Mantiuk tone mapping algorithm.
MantiukGamma = 1.0
MantiukSaturation = 0.0
MantiukScale = 0.85

# Indicator for activating the rounding of the image angles.
roundcorns = False

# Indicator for activating the histogram window.
showHist = True

# Indicator to make the Y axis of the histogram with a logarithmic scale.
logarithmHist = False

# Indicator for activating image rotation.
rotation = False
rotationValue = 0.0

# Indicator for activating the cropping of the image margins.
cropping = False
cropT = 0
cropL = 0
cropR = 0
cropB = 0

# It is used to read the images contained in the resource file qrc_DSuper8.py.


def readImgFromQrc(file):
	imgfile = QFile(file)
	if imgfile.open(QIODevice.ReadOnly):
		cvimg = imdecode(fromstring(imgfile.readAll(), dtype=uint8),
                   IMREAD_COLOR)
		imgfile.close()
	else:
		info("File " + file + " not found in qrc")

	return cvimg
